open Term

module M = Map.Make(String)

exception Not_unifiable

let emptysubst = M.empty	(* substitutia vida *)

let comp f g x = f (g x)

let rec print_term = function
  | Const s | Var s -> print_string s
  | Fun (f, tl) -> print_string f; print_char '('; print_term (List.hd tl);
    List.iter (fun t -> print_char ','; print_term t) (List.tl tl);
    print_char ')'

let mapterm m = 
  let rec mt = function
    | Const v as t -> t
    | Fun (f, tl) -> Fun (f, List.map mt tl)
    | Var vn as t -> try mt (M.find vn m) with Not_found -> t
  in mt

let print_mapterm t m = print_term (mapterm m t) (* tipareste termen substit. *)

let addsubst vn t m = 		(* adauga vn -> t la substitutia m *)
  let rec occurcheck vn = function	(* verifica sa nu fie circulara *)
    | Const _ -> ()
    | Fun (_, tl) -> List.iter (occurcheck vn) tl
    | Var vn1 -> if vn = vn1 then raise Not_unifiable
  in (occurcheck vn t; M.add vn t m)
			      
let rec unifmap m t1 t2 =	(* substitutia rezultanta prin unificare *)
  let r1 = mapterm m t1 and r2 = mapterm m t2 in match (r1, r2) with
    | (Const c1, Const c2) -> if c1 = c2 then m else raise Not_unifiable
    | (Const _, Fun _) | (Fun _, Const _) -> raise Not_unifiable
    | (Fun (f1, tl1), Fun (f2, tl2)) ->
      if f1 = f2 then List.fold_left2 unifmap m tl1 tl2
      else raise Not_unifiable
    | (Var vn, _) -> if r1 = r2 then m else addsubst vn r2 m
    | (_, Var vn) -> addsubst vn r1 m


